import "reflect-metadata";
import { Connector, FhModule } from "fh-forms-handler";
import { DynamicRelocatorExclusionRule, DynamicRelocatorExtensionRule } from 'fhdp-extenders';
export interface LiferayConfig {
    /** Defines if liferay is enabled or not */
    enabled: boolean;
    /** Local websocket addres For debug */
    localPumaWsURL: string;
    /** Remote websocket address */
    fhBaseUrl: string;
    /** Remote context path */
    fhContextPath: string;
}
export interface ExtenderConfig {
    extendFHML?: boolean;
    mobileMaxWidth?: number;
    enableMobileNavbar?: boolean;
    enableErrorBelowField?: boolean;
    enableDynamicFooterPosition?: boolean;
    enableCurrentMenuElementHighlight?: boolean;
    currentMenuElementHighlighBottomBorderStyle?: string;
    enableFHMLinTooltips?: boolean;
    enableSessionCounterRules?: boolean;
    enableCookieAlert?: boolean;
    additionalRules?: Array<DynamicRelocatorExclusionRule | DynamicRelocatorExtensionRule>;
}
export interface FhApplicationConfig {
    /** Defines registration of standard modules (FormsHandler and BasicControls) */
    registerStandardModules?: boolean;
    /** Defines registration of Charts module */
    registerChartsControls?: boolean;
    /** Defines registration of FHDP controls module*/
    registerFHDPControls?: boolean;
    /** Defines registration of additional modules*/
    additionalModules?: ({
        new (): FhModule;
    })[];
    /** Defines translations for connection lost message Default contains messages for polish and english languages */
    connectionLostTranslations?: {
        [key: string]: string;
    };
    /** Defines config for liferay If empty it will act as normal application */
    liferay?: LiferayConfig;
    /** Defines config for FHDPExtender If empty it will act as normal fh app without extensions */
    extensionsConfig?: ExtenderConfig;
}
declare class FhApplication {
    /** Instance of FhApplication */
    private static __instance;
    /** Current config */
    private __config;
    /** Instance of Liferay config */
    private __liferay?;
    /** Instance of connector */
    private __connector?;
    /**
     * Private and used internally
     * @param config main configuration
     */
    private constructor();
    /**
     * At first use must be called with `config` param, to properly init instance
     * Later uses can be without param, and it will return same instance as one inited at begin
     * @param config main configuration
     */
    static getInstance(config?: FhApplicationConfig): FhApplication;
    /**
     * Getter for connection lost translations
     */
    private get currentConnectionLostTranslation();
    private isJSON;
    /**
     * Getter for connector;
     * @returns Connector
     */
    getConnector(): Connector;
    /**
     * Registers new module Its alternative function to config node `additionalModules`
     * @param module an module that will be registered in fh app
     */
    registerModule(module: {
        new (): FhModule;
    }): void;
    /**
     * Registers callback for backend calls
     * @param name callback name (key)
     * @param callback actual callback function
     */
    registerCallback(name: string, callback: (...args: any[]) => void): Error;
    /**
     * Function for initialise extenders
     */
    private initExtenders;
    /**
     * Adds translation string
     */
    addTranslation(lang: any, key: any, translation: any): void;
    /**
     * get translation string
     */
    translate(key: any, args: any): string;
    /**
     * Initialise translations for connection lost
     */
    private initConnectionLostTranslations;
    /**
     * Main init function
     * @param context app context for sockets Default 'socketForms'
     */
    init(context?: string): void;
    /**
     * Edits classlist of element provided by id
     * @param id Id of DOM element
     * @param remove class name that should be removed from element
     * @param add class name that should be added in element
     */
    createCallbackById(id: string, remove?: string, add?: string): void;
    /**
     * Edits classlist of element wrapper provided by id
     * @param id Id of DOM element
     * @param remove class name that should be removed from element wrapper
     * @param add class name that should be added in element wrapper
     */
    createCallbackByWrapper(id: string, remove?: string, add?: string): void;
}
export { FhApplication };
